package blueprint

import (
	"bytes"
	"errors"
	"fmt"
	"os"
	"path/filepath"
	"text/template"

	"github.com/rs/zerolog/log"
	"gopkg.in/yaml.v3"
)

const BlueprintDefaultPath = "./"

var (
	BluePrintFileNames = []string{"Blueprint.yaml", "Blueprint.yml", "blueprint.yaml", "blueprint.yml"}
)

// BlueprintFile - описание модели файла.
type BlueprintFile struct {
	ProjectName     string           `yaml:"ProjectName"`
	Description     string           `yaml:"Description,omitempty"`
	Author          string           `yaml:"Author,omitempty"`
	ModulesSettings map[string]any   `yaml:"Modules"`
	Template        TemplateSettings `yaml:"Template,omitempty"`
	Vars            map[string]any   `yaml:"Vars,omitempty"`
}

func readFile(path string) (blueprint *BlueprintFile, err error) {
	var fileStat os.FileInfo

	if path == "" {
		path = BlueprintDefaultPath
	}

	var fullPath string
	for _, fileName := range BluePrintFileNames {
		fullPath = filepath.Join(path, fileName)

		fileStat, err = os.Stat(fullPath)
		if errors.Is(err, os.ErrNotExist) {
			continue
		}

		err = nil
		break
	}

	if os.IsNotExist(err) {
		return nil, fmt.Errorf("blueprint file not found")
	}

	if fileStat.Size() <= 0 {
		return nil, fmt.Errorf("blueprint file is empty")
	}

	file, err := os.ReadFile(fullPath)
	if err != nil {
		// Проверяем еще раз на всякий случай.
		if errors.Is(err, os.ErrNotExist) {
			return nil, fmt.Errorf("blueprint file not found")
		}

		return nil, fmt.Errorf("cannot read blueprint file: %w", err)
	}

	log.Info().Str("file", fullPath).Msg("Reading blueprint file")

	// Получаем базовый файл.
	if err = yaml.Unmarshal(file, &blueprint); err != nil {
		return nil, fmt.Errorf("cannot parsing blueprint file: %w", err)
	}

	// Преобразуем шаблонные значения в структуру
	t, err := template.New("blueprint").Parse(string(file))
	if err != nil {
		return nil, fmt.Errorf("cannot template blueprint file: %w", err)
	}

	buf := bytes.NewBuffer([]byte{})

	if err = t.Execute(buf, blueprint); err != nil {
		return nil, fmt.Errorf("cannot execute blueprint file: %w", err)
	}

	file = buf.Bytes()
	if err = yaml.Unmarshal(file, &blueprint); err != nil {
		return nil, fmt.Errorf("cannot reparsing blueprint file: %w", err)
	}

	return blueprint, err
}
