package blueprint

import (
	"fmt"
	"path/filepath"

	"penahub.gitlab.yandexcloud.net/pena-services/blueprint/blueprint/modules/env"
	"penahub.gitlab.yandexcloud.net/pena-services/blueprint/blueprint/modules/logger"
)

type BlueprintCreator struct {
	file     *BlueprintFile
	template *Template
	modules  []BlueprintModule
}

type BlueprintModule interface {
	Name() string
	SetSettings(settings any) error
	Execute() error
}

func NewBlueprintCreator(blueprintPath, projectName string) (*BlueprintCreator, error) {
	file, err := readFile(blueprintPath)

	if err != nil {
		return nil, err
	}

	if projectName != "" {
		file.ProjectName = projectName
	}

	blueprintModules := []BlueprintModule{
		logger.NewLoggerModule(),
		env.NewEnvModule(),
	}

	templateModules := make(map[string]BlueprintModule, 0)

	for _, module := range blueprintModules {
		if file.ModulesSettings[module.Name()] != nil {
			if err = module.SetSettings(file.ModulesSettings[module.Name()]); err != nil {
				return nil, fmt.Errorf("blueprintCreator: cannot set settings for module %v. %v", module.Name(), err)
			}
			templateModules[module.Name()] = module
		}
	}

	// get absolute paths
	absExecPath, err := filepath.Abs("./")
	if err != nil {
		return nil, fmt.Errorf("blueprintCreator: cannot get absolute path for execPath. %v", err)
	}

	template := NewTemplate(TemplateSettings{
		ExecPath:     absExecPath,
		TemplatePath: filepath.Join(blueprintPath, file.Template.TemplatePath),
		Vars:         file.Template.Vars,
		Modules:      templateModules,
	})

	template.SetVar("ProjectName", file.ProjectName)
	template.SetVar("Author", file.Author)
	template.SetVar("Description", file.Description)

	return &BlueprintCreator{
		template: template,
		file:     file,
		modules:  blueprintModules,
	}, err
}

func (r *BlueprintCreator) Create() error {
	if err := r.template.Execute(); err != nil {
		return fmt.Errorf("template execute. %v", err)
	}

	for _, module := range r.modules {
		if err := module.Execute(); err != nil {
			return fmt.Errorf("module execute (%v). %v", module.Name(), err)
		}
	}

	return nil
}
