package env

import (
	"fmt"
	"os/exec"
	"strings"

	"golang.org/x/text/cases"
	"golang.org/x/text/language"
	"gopkg.in/yaml.v3"
)

type EnvModule struct {
	settings Settings
}

type Settings struct {
	Vars []Var `yaml:"vars"`
}

type Var struct {
	Name     string `yaml:"name"`
	Type     string `yaml:"type"`
	Required bool   `yaml:"required,omitempty"`
	Default  string `yaml:"default"`
}

func (r Var) code() string {
	var fieldName, req, def string

	if r.Required {
		req = ",required"
	}

	if r.Default != "" {
		def = fmt.Sprintf(` envDefault:"%v"`, r.Default)
	}

	caser := cases.Title(language.English)

	for _, item := range strings.Split(r.Name, "_") {
		fieldName += caser.String(item)
	}

	return fmt.Sprintf("	%v %v `env:\"%v%v\"%v`", fieldName, r.Type, r.Name, req, def)
}

func NewEnvModule() *EnvModule {
	return &EnvModule{}
}

func (r *EnvModule) Name() string {
	return "env"
}

func (r *EnvModule) SetSettings(settings any) error {
	out, err := yaml.Marshal(settings)
	if err != nil {
		return err
	}

	if err = yaml.Unmarshal(out, &r.settings); err != nil {
		return err
	}

	// Проверяем, обязательные поля name и type
	for _, v := range r.settings.Vars {
		if v.Name == "" {
			return fmt.Errorf("name required for env.vars item")
		}

		if v.Type == "" {
			return fmt.Errorf("type required for env.vars item (%v)", v.Name)
		}
	}

	return nil
}

func (r *EnvModule) Execute() error {
	value := strings.Replace(r.Import(), `"`, "", -1)
	cmd := exec.Command("go", "get", value)

	return cmd.Run()
}

func (r *EnvModule) Import() string {
	return `"github.com/caarlos0/env/v8"`
}

func (r *EnvModule) Declaration(v string) string {
	if v == "" {
		return "//FIXME TEMPLATE ERROR: env.Declaration required with non-empty argument"
	}

	return fmt.Sprintf(`err = env.Parse(%v)
	if err != nil {
		panic(err)
	}`, v)
}

func (r *EnvModule) Struct() string {
	var varsString string

	for _, v := range r.settings.Vars {
		varsString += fmt.Sprintf("%v\r\n", v.code())
	}

	return fmt.Sprintf("type Config struct {\r\n%v}", varsString)
}
