package blueprint

import (
	"bytes"
	"fmt"
	"io/fs"
	"os"
	"os/exec"
	"path/filepath"
	"strings"
	"text/template"

	"github.com/rs/zerolog/log"
)

const (
	DefaultPath = "./template"
)

var (
	TemplateFormats = []string{"tmpl", "gotmpl"}
)

type TemplateSettings struct {
	ExecPath     string                     `yaml:"-"`
	TemplatePath string                     `yaml:"path"`
	Vars         map[string]any             `yaml:"vars,omitempty"`
	Modules      map[string]BlueprintModule `yaml:"-"`
}

type Template struct {
	settings *TemplateSettings
	queue    []string
}

func NewTemplate(settings TemplateSettings) *Template {
	if settings.Vars == nil {
		settings.Vars = make(map[string]any)
	}

	return &Template{
		settings: &settings,
		queue:    make([]string, 0),
	}
}

func (r *Template) SetVar(name string, value any) bool {
	if _, ok := r.settings.Vars[name]; ok {
		return false
	}

	r.settings.Vars[name] = value

	return true
}

func (r *Template) Execute() error {
	log.Info().Msg("template.Execute: started")
	err := filepath.WalkDir(r.settings.TemplatePath, r.walkFunc)
	if err != nil {
		return err
	}

	for _, path := range r.queue {
		if err = r.templateFile(path); err != nil {
			return err
		}
	}

	return r.execCommands()
}

func (r *Template) templateFile(templatePath string) error {
	// Create output path - replace template path to exec path
	sp := strings.Split(templatePath, "template")

	outputFile := r.settings.ExecPath + sp[1]

	// Create outputh path - template path
	t, err := template.New("filepath").Parse(outputFile)
	if err != nil {
		return fmt.Errorf("templateFile: parse filepath (%v). %v", templatePath, err)
	}

	buf := bytes.NewBuffer(nil)
	if err = t.Execute(buf, r.settings.Vars); err != nil {
		return fmt.Errorf("templateFile: execute filepath (%v). %v", templatePath, err)
	}

	outputFile = buf.String()

	// Create output path - replace file format to .go
	for _, format := range TemplateFormats {
		outputFile = strings.Replace(outputFile, "."+format, "", -1)
	}

	// Create file

	err = os.MkdirAll(filepath.Dir(outputFile), 0750)
	if err != nil {
		return fmt.Errorf("templateFile: make directory (%v). %v", filepath.Dir(outputFile), err)
	}

	file, err := os.Create(outputFile)
	if err != nil {
		return err
	}

	// Template file
	t, err = template.New(filepath.Base(templatePath)).ParseFiles(templatePath)
	if err != nil {
		return fmt.Errorf("templateFile: parse template (%v). %v", templatePath, err)
	}

	if err = t.Execute(file, r.settings); err != nil {
		return fmt.Errorf("templateFile: execute template (%v). %v", templatePath, err)
	}

	log.Info().Str("file", outputFile).Msg("templateFile: file generated")

	return nil
}

func (r *Template) execCommands() error {
	cmd := exec.Command("go", "mod", "init")

	log.Info().Msg("execCommands: go mod init")

	if err := cmd.Run(); err != nil {
		return fmt.Errorf("execCommands: go mod init. %v", err)
	}

	cmd = exec.Command("go", "mod", "tidy")

	log.Info().Msg("execCommands: go mod tidy")

	if err := cmd.Run(); err != nil {
		return fmt.Errorf("execCommands: go mod tidy. %v", err)
	}

	return nil
}

func (r *Template) walkFunc(path string, d fs.DirEntry, err error) error {
	if err != nil {
		return fmt.Errorf("walkFunc: %v", err)
	}

	info, err := d.Info()
	if err != nil {
		return fmt.Errorf("walkFunc: cannot get info about path (%v). %v", path, err)
	}

	if isTemplateFile(info) {
		absTemplatePath, err := filepath.Abs(path)
		if err != nil {
			return fmt.Errorf("walkFunc: cannot get absolut path for path (%v). %v", path, err)
		}

		r.queue = append(r.queue, absTemplatePath)
	}

	return nil
}

func isTemplateFile(info fs.FileInfo) bool {
	if info.IsDir() {
		return false
	}

	sp := strings.Split(info.Name(), ".")

	if len(sp) < 2 {
		return false
	}

	format := strings.ToLower(sp[len(sp)-1])

	for _, v := range TemplateFormats {
		if format == v {
			return true
		}
	}

	return false
}
